import 'dart:io';

import 'package:firebase_messaging/firebase_messaging.dart';
import 'package:ocg_app/ocg_app.dart';
import 'package:ocg_notification/src/models/notification.dart';
import 'package:ocg_notification/src/repository/notification.dart';
import 'package:ocg_notification/src/service/base.dart';
import 'package:flutter/foundation.dart';

class NotificationFirebaseService extends NotificationService {
  final FirebaseMessaging firebaseMessaging;
  final NotificationRepository notificationRepository;

  NotificationFirebaseService(this.notificationRepository)
      : this.firebaseMessaging = FirebaseMessaging();

  @override
  Future<AppResponse<bool>> subscribe() async {
    try {
      bool isCheck = await firebaseMessaging.requestNotificationPermissions();
      if (Platform.isIOS && !(isCheck ?? false)) {
        return AppResponse.named(error: "IOS permission not granted");
      }
      firebaseMessaging.onTokenRefresh.listen(((val) {
        notificationRepository.saveFirebaseToken(val);
      }));
      // Get Token from firebase
      String fcmToken = await firebaseMessaging.getToken();
      debugPrint("FCM $fcmToken");
      return notificationRepository.saveFirebaseToken(fcmToken);
    } catch (e) {
      return AppResponse.named(error: e.toString());
    }
  }

  @override
  Future<AppResponse<bool>> unsubscribe() async {
    try {
      String fcmToken = await firebaseMessaging.getToken();
      notificationRepository.deleteFirebaseToken(fcmToken);
      await firebaseMessaging.deleteInstanceID();
      return AppResponse.named(data: true);
    } catch (e) {
      return AppResponse.named(error: e.toString());
    }
  }

  @override
  Future<void> onNotification(NotificationCallback callback,
      {Function(Map<String, dynamic> message) backgroundMessageHandler}) async {
    debugPrint("Notification Service Listen");
    firebaseMessaging.configure(
      onBackgroundMessage: backgroundMessageHandler,
      onMessage: (Map<String, dynamic> message) async {
        debugPrint("on Message");
        callback(convertToNotification(message)
            .copyWith(source: AppNotificationSource.Message));
      },
      onLaunch: (Map<String, dynamic> message) async {
        debugPrint("on Launch");
        callback(convertToNotification(message)
            .copyWith(source: AppNotificationSource.Launch));
      },
      onResume: (Map<String, dynamic> message) async {
        debugPrint("on Resume");
        callback(convertToNotification(message)
            .copyWith(source: AppNotificationSource.Resume));
      },
    );
  }
}

AppNotification convertToNotification(temp) {
  var json = temp["data"] ?? temp ?? {};
  var notification = temp["notification"] ?? temp ?? {};
  return AppNotification.named(
      title: notification["title"] ?? json["title"],
      description: notification["body"] ?? json["body"],
      json: json);
}
